/* cmd_neonserv_unban.c - NeonServ v5.6
 * Copyright (C) 2011-2012  Philipp Kreil (pk910)
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License 
 * along with this program. If not, see <http://www.gnu.org/licenses/>. 
 */

#include "cmd_neonserv.h"

/*
* argv[0-*]    nick[,*auth[,*!*@mask[...]]]
*/
struct neonserv_cmd_unban_cache {
    struct ClientSocket *client, *textclient;
    struct UserNode *user;
    struct ChanNode *chan;
    struct Event *event;
    struct ModeBuffer *modeBuf;
    int provided_masks, done_masks, pending_whos, unbanned_masks;
};

static USERAUTH_CALLBACK(neonserv_cmd_unban_userauth_lookup);
static void neonserv_cmd_unban_nick(struct neonserv_cmd_unban_cache *cache, struct UserNode *user);
static void neonserv_cmd_unban_mask(struct neonserv_cmd_unban_cache *cache, char *mask);
static void neonserv_cmd_unban_finish(struct neonserv_cmd_unban_cache *cache);

CMD_BIND(neonserv_cmd_unban) {
    char *mask, *nextmask;
    struct ModeBuffer *modeBuf;
    modeBuf = initModeBuffer(client, chan);
    nextmask = merge_argv_char(argv, 0, argc, ',');
    struct neonserv_cmd_unban_cache *cache = malloc(sizeof(*cache));
    if (!cache) {
        printf_log("neonserv", LOG_ERROR, "%s:%d malloc() failed", __FILE__, __LINE__);
        return;
    }
    cache->client = client;
    cache->textclient = textclient;
    cache->user = user;
    cache->chan = chan;
    cache->event = event;
    cache->modeBuf = modeBuf;
    cache->done_masks = 0;
    cache->provided_masks = 0;
    cache->unbanned_masks = 0;
    while((mask = nextmask)) {
        nextmask = strstr(mask, ",");
        if(nextmask) {
            *nextmask = '\0';
            nextmask++;
        }
        cache->provided_masks++;
        if(is_valid_nick(mask)) {
            struct UserNode *cuser = getUserByNick(mask);
            if(!cuser) {
                cuser = createTempUserMask(mask);
				if(!cuser) {
					break; //internal bot error
				}
                cuser->flags |= USERFLAG_ISTMPUSER;
                get_userauth(cuser, module_id, neonserv_cmd_unban_userauth_lookup, cache);
                cache->pending_whos++;
            } else {
                neonserv_cmd_unban_nick(cache, cuser);
            }
        } else {
            neonserv_cmd_unban_mask(cache, mask);
        }
    }
    if(!cache->pending_whos)
        neonserv_cmd_unban_finish(cache);
}

static USERAUTH_CALLBACK(neonserv_cmd_unban_userauth_lookup) {
    struct neonserv_cmd_unban_cache *cache = data;
    cache->pending_whos--;
    if(user)
        neonserv_cmd_unban_nick(cache, user);
    else
        neonserv_cmd_unban_mask(cache, user_nick);
    if(!cache->pending_whos)
        neonserv_cmd_unban_finish(cache);
}

static void neonserv_cmd_unban_nick(struct neonserv_cmd_unban_cache *cache, struct UserNode *user) {
    int matches = 0;
    struct BanNode *ban;
    char usermask[NICKLEN+USERLEN+HOSTLEN+3];
    sprintf(usermask, "%s!%s@%s", user->nick, user->ident, user->host);
    for(ban = cache->chan->bans; ban; ban = ban->next) {
        if(!match(ban->mask, usermask)) {
            modeBufferUnban(cache->modeBuf, ban->mask);
            cache->unbanned_masks++;
            matches++;
        }
    }
    if(matches)
        cache->done_masks++;
}

static void neonserv_cmd_unban_mask(struct neonserv_cmd_unban_cache *cache, char *mask) {
    char banmask[NICKLEN+USERLEN+HOSTLEN+3];
    int matches = 0;
    struct BanNode *ban;
    mask = make_banmask(mask, banmask);
    for(ban = cache->chan->bans; ban; ban = ban->next) {
        if(!match(mask, ban->mask)) {
            modeBufferUnban(cache->modeBuf, ban->mask);
            cache->unbanned_masks++;
            matches++;
        }
    }
    if(matches)
        cache->done_masks++;
    else {
        for(ban = cache->chan->bans; ban; ban = ban->next) {
            if(!match(ban->mask, mask)) {
                reply(cache->textclient, cache->user, "NS_DELBAN_BANNED_BY", mask, ban->mask);
                break;
            }
        }
    }
}

static void neonserv_cmd_unban_finish(struct neonserv_cmd_unban_cache *cache) {
    freeModeBuffer(cache->modeBuf);
    if(cache->done_masks == cache->provided_masks)
        reply(cache->textclient, cache->user, "NS_UNBAN_DONE", cache->unbanned_masks, cache->chan->name);
    else
        reply(cache->textclient, cache->user, "NS_UNBAN_FAIL", cache->client->user->nick);
    if(cache->done_masks)
        logEvent(cache->event);
    free(cache);
}

